@@ -3,7 +3,7 @@ from .normalise import Normalise
 # Metadata
 __doc__ = "Package to help with normalizing data needed for the platform!"
 # Github CI/CD greps for the version number.
-__version__ = "1.1.4"
-__author__ = "Realstats"
+__version__ = "1.2"
+__author__ = "Michiel Masselink"
 __email__ = "m.masselink@yahoo.com"
 __license__ = "MIT"
