@@ -7,6 +7,7 @@ from difflib import SequenceMatcher
 from math import floor
 from time import time
 import pandas as pd
+pd.options.mode.chained_assignment = None  # default='warn'
 import numpy as np
 
 
@@ -140,7 +141,14 @@ class Normalise:
         global_action = False
         global_verbose = "to_console"
 
+        # Which rows to drop in one go
+        drop_array = []
+        
+        # Which rows to np.nan in one go
+        nan_array = []
+
         error_file = None
+
         # regex to match the string to_file in between quotes or double quotes
         regex = re.compile(r'\"(to_file)\"|\'(to_file)\'')
         if regex.search(str(rules)):
@@ -161,10 +169,12 @@ class Normalise:
                 action = rules[j]["action"]
 
             if action == "np.nan":
-                rows[i] = np.nan
+                # Which rows to np.nan later on in one go
+                nan_array.append(i)
 
             if action == "drop":
-                dataframe.drop(i, inplace=True)
+                # Which rows to drop later onin one go
+                drop_array.append(i)
 
             verbose = global_verbose
             if "verbose" in rules[j]:
@@ -273,7 +283,10 @@ class Normalise:
 
             if "range" in rules[j]:
                 for i, value in column_values.items():
-                    if not (float(rules[j]["range"][0]) <= float(value) <= float(rules[j]["range"][1])):
+                    try:
+                        if not (float(rules[j]["range"][0]) <= float(value) <= float(rules[j]["range"][1])):
+                            handle_mismatch(i, value, column_values)
+                    except ValueError:
                         handle_mismatch(i, value, column_values)
 
             if "type" in rules[j]:
@@ -331,10 +344,10 @@ class Normalise:
                             handle_mismatch(i, value, column_values)
 
                 if rules[j]["type"] == "postal_code":
-                    column_values.str.replace(
+                    column_values.astype(str).str.replace(
                         r"[^a-zA-Z0-9]+", "", regex=True).str.upper()
-                    for i, value in column_values.items():
 
+                    for i, value in column_values.items():
                         if not re.match(r"^[1-9][0-9]{3}\s?[a-zA-Z]{2}$", str(value)):
                             handle_mismatch(i, value, column_values)
 
@@ -371,7 +384,15 @@ class Normalise:
                 j -= 1
                 continue
 
+            # Bulk drop rows
+            dataframe.drop(drop_array, inplace=True)
+
+            # Make all the indexes inside the np.nan array nan values
+            column_values.loc[nan_array] = np.nan
+
             # Update final dataframe with cached values
             dataframe[rules[j]["column"]].update(column_values)
+    
+
             errors_found = False
         return dataframe
